==> ada-cloud-util-0.3.8.tgz: package/boost/controller/manager.js <==
const Metadata = require("./../../metadata");
const Router = require("./router");
const { Result, ErrorResult, NofoundResult } = require("./../../result");
const BoostProvider = require("../provider");
const { debug } = require("./../../const");

const ControllerManager = {
    router: new Router(),
    set(clazz) {
        let main = Metadata.getClassMetadataExtends(clazz.prototype),
            methods = Metadata.getMethodMetadataExtends(clazz.prototype);
        let basePath = '';
        if (main && main.controller) {
            basePath = main.controller.path || '';
        }
        Reflect.ownKeys(methods).forEach(key => {
            let actionInfo = methods[key];
            let { path = '', method = 'get' } = actionInfo;
            let _path = `/${method}/${basePath}/${path}`.replace(/[\/]+/g, '/');
            this.router.add(_path, {
                action: key,
                actionInfo,
                controller: clazz
            });
        });
    },
    excute(path, ctx, next) {
        let r = this.router.check(path);
        if (r && r.found) {
            ctx.parameters = r.map;
            let { action, controller, actionInfo } = r.callback;
            let controllerInstance = BoostProvider.getEntity(controller);
            if (controllerInstance[action]) {
                controllerInstance.requestContext = ctx;
                let _ps = Promise.resolve();
                _ps = _ps.then(() => controllerInstance.beforeExcute({ context: ctx, action: actionInfo }));
                _ps = _ps.then(result => {
                    if (result !== false) {
                        return Promise.resolve().then(() => controllerInstance[action](ctx)).then(a => {
                            if (a && a instanceof Result) {
                                ctx.set("Content-Type", "application/json");
                                ctx.response.body = a.getResponseData();
                            } else {
                                ctx.response.body = a;
                            }
                        });
                    }
                });
                _ps = _ps.then(() => controllerInstance.afterExcute({ context: ctx, action: actionInfo }));
                _ps = _ps.then(() => {
                    controllerInstance.requestContext = null;
                    return next();
                }).catch(a => {
                    debug(a);
                    return Promise.resolve().then(() => controllerInstance.actionExcuteError({ context: ctx, action: actionInfo, error: a })).then(result => {
                        if (result !== false) {
                            ctx.set("Content-Type", "application/json");
                            ctx.response.body = new ErrorResult(a.toString()).getResponseData();
                            controllerInstance.requestContext = null;
                            return next();
                        } else {
                            controllerInstance.requestContext = null;
                            return next();
                        }
                    }).catch(a => {
                        debug(a);
                        ctx.set("Content-Type", "application/json");
                        ctx.response.body = new ErrorResult(a.toString()).getResponseData();
                        controllerInstance.requestContext = null;
                        return next();
                    });
                });
                return _ps;
            } else {
                return Promise.resolve().then(() => controllerInstance.actionNotFound({ context: ctx, action: actionInfo })).then(result => {
                    if (result !== false) {
                        controllerInstance.requestContext = null;
                        ctx.set("Content-Type", "application/json");
                        ctx.response.body = new NofoundResult('action can not found').getResponseData();
                        return next();
                    } else {
                        controllerInstance.requestContext = null;
                        return next();
                    }
                }).catch(a => {
                    debug(a);
                    return Promise.resolve().then(() => controllerInstance.actionExcuteError({ context: ctx, action: actionInfo, error: a })).then(result => {
                        if (result !== false) {
                            ctx.set("Content-Type", "application/json");
                            ctx.response.body = new ErrorResult(a.toString()).getResponseData();
                            controllerInstance.requestContext = null;
                            return next();
                        } else {
                            controllerInstance.requestContext = null;
                            return next();
                        }
                    }).catch(a => {
                        debug(a);
                        ctx.set("Content-Type", "application/json");
                        ctx.response.body = new ErrorResult(a.toString()).getResponseData();
                        controllerInstance.requestContext = null;
                        return next();
                    });
                });
            }
        }
    }
}

module.exports = ControllerManager;

==> ada-cloud-util-0.3.10.tgz: package/boost/controller/manager.js <==
const Metadata = require("./../../metadata");
const Router = require("./router");
const { Result, ErrorResult, NofoundResult } = require("./../../result");
const BoostProvider = require("../provider");
const { debug } = require("./../../const");

const ControllerManager = {
    router: new Router(),
    set(clazz) {
        let main = Metadata.getClassMetadataExtends(clazz.prototype),
            methods = Metadata.getMethodMetadataExtends(clazz.prototype);
        let basePath = '';
        if (main && main.controller) {
            basePath = main.controller.path || '';
        }
        Reflect.ownKeys(methods).forEach(key => {
            let actionInfo = methods[key];
            let { path = '', method = 'get' } = actionInfo;
            let _path = `/${method}/${basePath}/${path}`.replace(/[\/]+/g, '/');
            this.router.add(_path, {
                action: key,
                actionInfo,
                controller: clazz
            });
        });
    },
    excute(path, ctx, next) {
        let r = this.router.check(path);
        if (r && r.found) {
            ctx.parameters = r.map;
            let { action, controller, actionInfo } = r.callback;
            let controllerInstance = BoostProvider.getEntity(controller);
            if (controllerInstance[action]) {
                controllerInstance.requestContext = ctx;
                let _ps = Promise.resolve();
                _ps = _ps.then(() => controllerInstance.beforeExcute({ context: ctx, action: actionInfo }));
                _ps = _ps.then(result => {
                    if (result !== false) {
                        return Promise.resolve().then(() => controllerInstance[action](ctx)).then(a => {
                            if (a && a instanceof Result) {
                                ctx.set("Content-Type", "application/json");
                                ctx.response.body = a.getResponseData();
                            } else {
                                ctx.response.body = a;
                            }
                        });
                    }
                });
                _ps = _ps.then(() => controllerInstance.afterExcute({ context: ctx, action: actionInfo }));
                _ps = _ps.then(() => {
                    controllerInstance.requestContext = null;
                    return next();
                }).catch(a => {
                    debug(a);
                    return Promise.resolve().then(() => controllerInstance.actionExcuteError({ context: ctx, action: actionInfo, error: a })).then(result => {
                        if (result !== false) {
                            ctx.set("Content-Type", "application/json");
                            ctx.response.body = new ErrorResult(a).getResponseData();
                            controllerInstance.requestContext = null;
                            return next();
                        } else {
                            controllerInstance.requestContext = null;
                            return next();
                        }
                    }).catch(a => {
                        debug(a);
                        ctx.set("Content-Type", "application/json");
                        ctx.response.body = new ErrorResult(a).getResponseData();
                        controllerInstance.requestContext = null;
                        return next();
                    });
                });
                return _ps;
            } else {
                return Promise.resolve().then(() => controllerInstance.actionNotFound({ context: ctx, action: actionInfo })).then(result => {
                    if (result !== false) {
                        controllerInstance.requestContext = null;
                        ctx.set("Content-Type", "application/json");
                        ctx.response.body = new NofoundResult('action can not found').getResponseData();
                        return next();
                    } else {
                        controllerInstance.requestContext = null;
                        return next();
                    }
                }).catch(a => {
                    debug(a);
                    return Promise.resolve().then(() => controllerInstance.actionExcuteError({ context: ctx, action: actionInfo, error: a })).then(result => {
                        if (result !== false) {
                            ctx.set("Content-Type", "application/json");
                            ctx.response.body = new ErrorResult(a).getResponseData();
                            controllerInstance.requestContext = null;
                            return next();
                        } else {
                            controllerInstance.requestContext = null;
                            return next();
                        }
                    }).catch(a => {
                        debug(a);
                        ctx.set("Content-Type", "application/json");
                        ctx.response.body = new ErrorResult(a).getResponseData();
                        controllerInstance.requestContext = null;
                        return next();
                    });
                });
            }
        }
    }
}

module.exports = ControllerManager;
